/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * A table with graph with more features.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgtablewithgraph.h"
#include "skgservices.h"
#include "skgtraces.h"
#include "skgmainpanel.h"
#include "skggraphicsscene.h"
#include <skgcolorbutton.h>

#include <ksavefile.h>
#include <kmenu.h>
#include <kfiledialog.h>
#include <kstringhandler.h>
#include <kstandarddirs.h>

#include <QtGui/QHeaderView>
#include <QtGui/QGraphicsLineItem>
#include <QDomDocument>
#include <QRegExp>
#include <QPrinter>
#include <QPainter>
#include <QTableWidgetItem>
#include <QDesktopServices>
#include <QTimer>
#include <QScriptEngine>

#include <math.h>

/**
  * Data identifier for value
  */
static const int DATA_VALUE = 12;
/**
  * Data identifier for color
  */
static const int DATA_COLOR_H = 11;
/**
  * Data identifier for color
  */
static const int DATA_COLOR_S = 12;
/**
  * Data identifier for color
  */
static const int DATA_COLOR_V = 13;
/**
  * Box size
  */
static const int BOX_SIZE = 120;
/**
  * Box margin
  */
static const int BOX_MARGIN = 10;

Qt::SortOrder SKGTableWithGraph::m_sortOrder = Qt::AscendingOrder;
int SKGTableWithGraph::m_sortColumn = 0;

SKGTableWithGraph::SKGTableWithGraph(QWidget *parent)
    : QWidget(parent), m_scene(NULL), m_additionalInformation(NONE), m_nbVirtualColumns(0),
      m_selectable(true), m_toolBarVisible(true), m_limitVisible(true), m_linearRegressionVisible(true), m_legendVisible(false), m_graphVisible(true),
      m_mainMenu(NULL),
      m_indexSum(-1), m_indexAverage(-1), m_indexMin(-1), m_indexLinearRegression(-1)
{
    ui.setupUi(this);
    ui.kTextEdit->hide();

    ui.kDisplayMode->addItem(KIcon("skg-chart-bar-stacked"), i18nc("Noun, a type of graph, with bars stacked upon each other", "Stack"));
    ui.kDisplayMode->addItem(KIcon("skg-chart-bar"), i18nc("Noun, a type of graph, with bars placed besides each other", "Histogram"));
    ui.kDisplayMode->addItem(KIcon("skg-chart-pie"), i18nc("Noun, a type of graph that looks like a sliced pie", "Pie"));
    ui.kDisplayMode->addItem(KIcon("skg-chart-ring"), i18nc("Noun, a type of graph that looks like concentric slices of a pie (a la filelight)", "Concentric pie"));
    ui.kDisplayMode->addItem(KIcon("skg-chart-scatter"), i18nc("Noun, a type of graph with only points", "Point"));
    ui.kDisplayMode->addItem(KIcon("skg-chart-line"), i18nc("Noun, a type of graph with only lines", "Line"));

    m_timer.setSingleShot(true);
    connect(&m_timer, SIGNAL(timeout()), this, SLOT(refresh()));

    m_timerRedraw.setSingleShot(true);
    connect(&m_timerRedraw, SIGNAL(timeout()), this, SLOT(redrawGraph()));

    //Build contextual menu
    ui.kTable->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(ui.kTable, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(showMenu(const QPoint&)));

    m_mainMenu = new KMenu(ui.kTable);

    QAction* actExport = m_mainMenu->addAction(KIcon("document-export"), i18nc("Noun, user action", "Export..."));
    connect(actExport, SIGNAL(triggered(bool)), this, SLOT(onExport()));

    QAction* resetColors = m_mainMenu->addAction(KIcon("format-fill-color"), i18nc("Noun, user action", "Reset default colors"));
    connect(resetColors, SIGNAL(triggered(bool)), this, SLOT(onResetColors()));

    m_actShowToolBar = getGraphContextualMenu()->addAction(i18nc("Noun, user action", "Show bottom tool bar"));
    if(m_actShowToolBar) {
        m_actShowToolBar->setCheckable(true);
        m_actShowToolBar->setChecked(m_toolBarVisible);
        connect(m_actShowToolBar, SIGNAL(triggered(bool)), this, SLOT(onSwitchToolBarVisibility()));
    }

    m_actShowLimits = getGraphContextualMenu()->addAction(i18nc("Noun, user action", "Show limits and average"));
    if(m_actShowLimits) {
        m_actShowLimits->setCheckable(true);
        m_actShowLimits->setChecked(m_limitVisible);
        connect(m_actShowLimits, SIGNAL(triggered(bool)), this, SLOT(onSwitchLimits()));
    }

    m_actShowLinearRegression = getGraphContextualMenu()->addAction(i18nc("Noun, user action", "Show tendency line"));
    if(m_actShowLinearRegression) {
        m_actShowLinearRegression->setCheckable(true);
        m_actShowLinearRegression->setChecked(m_linearRegressionVisible);
        connect(m_actShowLinearRegression, SIGNAL(triggered(bool)), this, SLOT(onSwitchLinearRegression()));
    }

    m_actShowLegend = getGraphContextualMenu()->addAction(i18nc("Noun, user action", "Show legend"));
    if(m_actShowLegend) {
        m_actShowLegend->setCheckable(true);
        m_actShowLegend->setChecked(m_legendVisible);
        connect(m_actShowLegend, SIGNAL(triggered(bool)), this, SLOT(onSwitchLegend()));
    }

    //Set headers parameters
    QHeaderView* verticalHeader = ui.kTable->verticalHeader();
    if(verticalHeader)  verticalHeader->hide();

    ui.kTable->setSortingEnabled(false);    //sort must be enable for refresh method
    QHeaderView* horizontalHeader = ui.kTable->horizontalHeader();
    if(horizontalHeader) {
        horizontalHeader->setResizeMode(QHeaderView::ResizeToContents);
        horizontalHeader->show();
        horizontalHeader->setSortIndicatorShown(true);
        horizontalHeader->setSortIndicator(m_sortColumn, m_sortOrder);
        connect(horizontalHeader, SIGNAL(sortIndicatorChanged(int, Qt::SortOrder)), this, SLOT(refresh()));
    }
    ui.kTable->verticalHeader()->setDefaultSectionSize(ui.kTable->verticalHeader()->minimumSectionSize());

    connect(ui.kDisplayMode, SIGNAL(currentIndexChanged(int)), this, SLOT(redrawGraphDelayed()), Qt::QueuedConnection);
    connect(ui.kAllPositive, SIGNAL(stateChanged(int)), this, SLOT(redrawGraphDelayed()), Qt::QueuedConnection);
    connect(ui.graphicView, SIGNAL(resized()), this, SLOT(redrawGraphDelayed()), Qt::QueuedConnection);
}


SKGTableWithGraph::~SKGTableWithGraph()
{
    if(m_scene) {
        delete m_scene;
        m_scene = NULL;
    }

    m_mainMenu = NULL;
    m_actShowToolBar = NULL;
    m_actShowLimits = NULL;
    m_actShowLinearRegression = NULL;
}

void SKGTableWithGraph::setToolBarVisible(bool iVisibility)
{
    m_toolBarVisible = iVisibility;
    ui.kGraphTypeFrm->setVisible(m_toolBarVisible);
    if(m_actShowToolBar) m_actShowToolBar->setChecked(m_toolBarVisible);
}

bool SKGTableWithGraph::isToolBarVisible() const
{
    return m_toolBarVisible;
}

void SKGTableWithGraph::onSwitchToolBarVisibility()
{
    setToolBarVisible(!isToolBarVisible());
}

void SKGTableWithGraph::onSwitchLimits()
{
    m_limitVisible = !m_limitVisible;
    redrawGraphDelayed();
}

void SKGTableWithGraph::onSwitchLegend()
{
    m_legendVisible = !m_legendVisible;
    redrawGraphDelayed();
}

void SKGTableWithGraph::onSwitchLinearRegression()
{
    m_linearRegressionVisible = !m_linearRegressionVisible;
    redrawGraphDelayed();
}

KMenu* SKGTableWithGraph::getTableContextualMenu() const
{
    return m_mainMenu;
}

KMenu* SKGTableWithGraph::getGraphContextualMenu() const
{
    return ui.graphicView->getContextualMenu();
}

void SKGTableWithGraph::showMenu(const QPoint& pos)
{
    if(m_mainMenu) m_mainMenu->popup(ui.kTable->mapToGlobal(pos));
}

QTableWidget* SKGTableWithGraph::table() const
{
    return ui.kTable;
}

SKGStringListList SKGTableWithGraph::getTable()
{
    //Build table
    SKGStringListList table;

    //Get header names
    int nb = ui.kTable->columnCount();
    QStringList cols;
    for(int i = 0; i < nb; i++) {
        cols.append(ui.kTable->horizontalHeaderItem(i)->text());
    }
    table.append(cols);

    //Get content
    int nb2 = ui.kTable->rowCount();
    for(int i = 0; i < nb2; i++) {
        QStringList row;

        for(int j = 0; j < nb; j++) {
            SKGColorButton* button = qobject_cast<SKGColorButton*>(ui.kTable->cellWidget(i, j));
            if(button) row.append(button->text());
            else row.append(ui.kTable->item(i, j)->text());
        }
        table.append(row);
    }
    return table;
}
QString SKGTableWithGraph::getState()
{
    SKGTRACEIN(10, "SKGTableWithGraph::getState");
    QDomDocument doc("SKGML");
    QDomElement root = doc.createElement("parameters");
    doc.appendChild(root);

    if(ui.graph_widget->isVisible() && ui.table_widget->isVisible()) root.setAttribute("splitterState", QString(ui.splitter->saveState().toHex()));
    root.setAttribute("graphMode", SKGServices::intToString((int) getGraphType()));
    root.setAttribute("allPositive", ui.kAllPositive->checkState() == Qt::Checked ? "Y" : "N");
    root.setAttribute("filter", ui.kFilterEdit->text());
    root.setAttribute("bottomToolBarVisible", isToolBarVisible() ? "Y" : "N");
    root.setAttribute("limitVisible", m_limitVisible ? "Y" : "N");
    root.setAttribute("linearRegressionVisible", m_linearRegressionVisible ? "Y" : "N");
    root.setAttribute("legendVisible", m_legendVisible ? "Y" : "N");
    QMapIterator<QString, QColor> i(m_mapTitleColor);
    while(i.hasNext()) {
        i.next();
        QDomElement color = doc.createElement("color");
        root.appendChild(color);
        color.setAttribute("key", i.key());
        color.setAttribute("value", i.value().name());
    }

    QHeaderView* horizontalHeader = ui.kTable->horizontalHeader();
    root.setAttribute("sortOrder", SKGServices::intToString(horizontalHeader->sortIndicatorOrder()));
    root.setAttribute("sortColumn", SKGServices::intToString(horizontalHeader->sortIndicatorSection()));
    root.setAttribute("graphicViewState", ui.graphicView->getState());
    root.setAttribute("web", ui.kTextEdit->getState());

    return doc.toString();
}

void SKGTableWithGraph::setState(const QString& iState)
{
    SKGTRACEIN(10, "SKGTableWithGraph::setState");
    QDomDocument doc("SKGML");
    doc.setContent(iState);
    QDomElement root = doc.documentElement();

    m_mapTitleColor.clear();

    QDomNodeList colors = root.elementsByTagName("color");
    int nb = colors.count();
    for(int i = 0; i < nb; ++i) {
        QDomElement c = colors.at(i).toElement();
        m_mapTitleColor[c.attribute("key")] = QColor(c.attribute("value"));
    }

    QString splitterState = root.attribute("splitterState");
    if(!splitterState.isEmpty()) ui.splitter->restoreState(QByteArray::fromHex(splitterState.toAscii()));
    QString graphMode = root.attribute("graphMode");
    QString allPositive = root.attribute("allPositive");
    QString bottomToolBarVisible = root.attribute("bottomToolBarVisible");
    QString limitVisibleString = root.attribute("limitVisible");
    QString legendVisibleString = root.attribute("legendVisible");
    QString linearRegressionVisibleString = root.attribute("linearRegressionVisible");
    QString sortOrder = root.attribute("sortOrder");
    QString sortColumn = root.attribute("sortColumn");
    QString graphicViewState = root.attribute("graphicViewState");
    QString web = root.attribute("web");

    //Defaut value in case of reset
    if(graphMode.isEmpty()) graphMode = "0";
    if(allPositive.isEmpty()) allPositive = "N";
    if(bottomToolBarVisible.isEmpty()) bottomToolBarVisible = "Y";
    if(limitVisibleString.isEmpty()) limitVisibleString = "Y";
    if(legendVisibleString.isEmpty()) legendVisibleString = "N";
    if(linearRegressionVisibleString.isEmpty()) linearRegressionVisibleString = "Y";
    if(sortOrder.isEmpty()) sortOrder = '0';
    if(sortColumn.isEmpty()) sortColumn = '0';

    //Set
    setGraphType(SKGTableWithGraph::HISTOGRAM);
    ui.kDisplayMode->setCurrentIndex(1);
    setGraphType((SKGTableWithGraph::GraphType) SKGServices::stringToInt(graphMode));
    ui.kAllPositive->setCheckState((allPositive == "Y" ? Qt::Checked : Qt::Unchecked));
    ui.kFilterEdit->setText(root.attribute("filter"));
    setToolBarVisible(bottomToolBarVisible == "Y");
    m_limitVisible = (limitVisibleString == "Y");
    if(m_actShowLimits) m_actShowLimits->setChecked(m_limitVisible);
    m_legendVisible = (legendVisibleString == "Y");
    if(m_actShowLegend) m_actShowLegend->setChecked(m_legendVisible);
    m_linearRegressionVisible = (linearRegressionVisibleString == "Y");
    if(m_actShowLinearRegression) m_actShowLinearRegression->setChecked(m_linearRegressionVisible);
    ui.kTable->setColumnCount(SKGServices::stringToInt(sortColumn) + 1);
    ui.kTable->horizontalHeader()->setSortIndicator(SKGServices::stringToInt(sortColumn), (Qt::SortOrder) SKGServices::stringToInt(sortOrder));
    ui.graphicView->setState(graphicViewState);
    ui.kTextEdit->setState(web);
}

void SKGTableWithGraph::onFilterModified()
{
    m_timer.start(300);
}

void SKGTableWithGraph::setData(const SKGStringListList& iData, SKGServices::SKGUnitInfo iPrimaryUnit,
                                SKGServices::SKGUnitInfo iSecondaryUnit,
                                DisplayAdditionalFlag iAdditionalInformation,
                                int iNbVirtualColumn)
{
    SKGTRACEIN(10, "SKGTableWithGraph::setData");
    m_data = iData;
    m_primaryUnit = iPrimaryUnit;
    m_secondaryUnit = iSecondaryUnit;
    m_additionalInformation = iAdditionalInformation;
    m_nbVirtualColumns = iNbVirtualColumn;

    refresh();
}

SKGTableWithGraph::DisplayAdditionalFlag SKGTableWithGraph::getAdditionalDisplayMode() const
{
    return m_additionalInformation;
}

bool SKGTableWithGraph::listSort(const QStringList &s1, const QStringList &s2)
{
    if(m_sortColumn >= s1.count()) m_sortColumn = s1.count() - 1;
    if(m_sortColumn >= 0) {
        QString v1 = s1.at(m_sortColumn);
        QString v2 = s2.at(m_sortColumn);
        if(m_sortColumn == 0) {
            int v = KStringHandler::naturalCompare(v1, v2, Qt::CaseInsensitive);
            return (m_sortOrder ? v > 0 : v < 0);
        }

        double vd1 = SKGServices::stringToDouble(v1);
        double vd2 = SKGServices::stringToDouble(v2);
        return (m_sortOrder ? vd1 > vd2 : vd1 < vd2);
    } else return false;
}

void SKGTableWithGraph::refresh()
{
    SKGTRACEIN(10, "SKGTableWithGraph::refresh");
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

    //Set parameters for static method
    QHeaderView* horizontalHeader = ui.kTable->horizontalHeader();
    m_sortOrder = horizontalHeader->sortIndicatorOrder();
    m_sortColumn = horizontalHeader->sortIndicatorSection();

    SKGStringListList groupedTable = m_data;
    int nbCols = -1;
    if(groupedTable.count()) nbCols = groupedTable.at(0).count();
    int nbRealCols = nbCols;

    //Create filtered table
    {
        //Build list of list of words
        QStringList words = SKGServices::splitCSVLine(ui.kFilterEdit->text(), ' ', true);
        int nbwords = words.count();

        QList<QStringList> listWords;
        QStringList currentList;
        for(int i = 0; i < nbwords; ++i) {
            QString word = words.at(i);
            bool isWordStartingByPlus = word.startsWith(QLatin1String("+"));
            if(isWordStartingByPlus) {
                word = word.right(word.length() - 1);
                if(i != 0) {
                    listWords.push_back(currentList);
                    currentList.clear();
                }
            }
            currentList.push_back(word);
        }
        listWords.push_back(currentList);

        //Check all lines
        int nblists = listWords.count();
        if(nblists && nbCols) {
            for(int i = groupedTable.count() - 1; i >= 1; --i) {  //The first line is not filtered because it's the title
                QStringList line = groupedTable.at(i);

                //Get title of the line
                QString val = line.at(0);

                //Filtered
                bool ok = false;
                for(int l = 0; !ok && l < nblists; ++l) {
                    QStringList words = listWords.at(l);
                    int nbwords = words.count();

                    bool validateAllWords = true;
                    for(int w = 0; validateAllWords && w < nbwords; ++w) {
                        validateAllWords = val.contains(words.at(w) , Qt::CaseInsensitive);
                    }
                    ok |= validateAllWords;
                }

                if(!ok) groupedTable.removeAt(i);
            }
        }
    }

    //Initialise sumRows
    int nblines = groupedTable.count();
    m_sumRows.clear();
    for(int j = 0; j < nblines; ++j) {
        m_sumRows.push_back(false);
    }

    //Compute sums line
    if(m_additionalInformation & SUM) {
        QStringList sums;
        sums.push_back("");
        for(int i = 1; i < nbCols; ++i) {
            double sum = 0;
            for(int j = 1; j < nblines; ++j) {
                QString valstring = groupedTable.at(j).at(i);
                if(!valstring.isEmpty()) sum += SKGServices::stringToDouble(valstring);
            }
            sums.push_back(SKGServices::doubleToString(sum));
        }
        groupedTable.push_back(sums);
        m_sumRows.push_back(true);
        ++nblines;
    }

    //Compute sub sums lines
    if(m_additionalInformation & SUM && m_sortColumn == 0 && m_sortOrder == Qt::AscendingOrder) {
        //Create a list of sums lines associted to the index where to add them
        QMap<int, QStringList> sums;
        for(int i = 1; i < nbCols; ++i) {
            QString previousHeaderSum;
            double sum = 0;
            int nbElem = 0;

            for(int j = 1; j < nblines - 1; ++j) {
                QString currentHeaderSum = groupedTable.at(j).at(0);
                int posSep = currentHeaderSum.indexOf(OBJECTSEPARATOR);
                if(posSep != -1) currentHeaderSum = currentHeaderSum.left(posSep);

                if(currentHeaderSum != previousHeaderSum) {
                    //Add this sum
                    if(nbElem > 1) {
                        QStringList thisSum;
                        if(i == 1) thisSum.push_back(previousHeaderSum);
                        else thisSum = sums[j];
                        thisSum.push_back(SKGServices::doubleToString(sum));

                        sums[j] = thisSum;
                    }

                    //Reset sum
                    sum = 0;
                    nbElem = 0;
                }
                previousHeaderSum = currentHeaderSum;

                QString valstring = groupedTable.at(j).at(i);
                if(!valstring.isEmpty()) sum += SKGServices::stringToDouble(valstring);
                ++nbElem;
            }

            //Add last sum
            if(nbElem > 1) {
                QStringList thisSum;
                if(i == 1) thisSum.push_back(previousHeaderSum);
                else thisSum = sums[nblines-1];
                thisSum.push_back(SKGServices::doubleToString(sum));
                sums[nblines-1] = thisSum;
            }
        }

        //Add all sums in table
        QList<int> keys = sums.keys();
        qSort(keys.begin(), keys.end());
        int nbkey = keys.count();
        for(int i = nbkey - 1; i >= 0; --i) {
            int key = keys.at(i);
            groupedTable.insert(key, sums[key]);
            m_sumRows.insert(key, true);
            ++nblines;
        }
    }

    //Compute sum and average column
    m_indexSum = -1;
    m_indexAverage = -1;
    m_indexMin = -1;
    m_indexLinearRegression = -1;
    if(nbCols > 2 && (m_additionalInformation & SUM || m_additionalInformation & AVERAGE || m_additionalInformation & LIMITS)) {
        SKGTRACEIN(10, "SKGTableWithGraph::refresh-computation");
        //Add title
        QStringList newLine = groupedTable.at(0);
        if(m_additionalInformation & SUM) {
            m_indexSum = newLine.count();
            newLine.push_back(i18nc("Noun, the numerical sum of a list of values", "Sum"));
        }
        if(m_additionalInformation & AVERAGE) {
            m_indexAverage = newLine.count();
            newLine.push_back(i18nc("Noun, the numerical average of a list of values", "Average"));
        }
        if(m_additionalInformation & LIMITS) {
            m_indexMin = newLine.count();
            newLine.push_back(i18nc("Noun, the minimum value of a list of values", "Min"));
            newLine.push_back(i18nc("Noun, the maximum value of a list of values", "Max"));
        }

        m_indexLinearRegression = newLine.count();
        newLine.push_back(i18nc("Noun", "Tendency line"));

        groupedTable.replace(0, newLine);

        for(int i = 1; i < nblines; ++i) {
            QStringList newLine = groupedTable.at(i);
            double sumy = 0;
            double sumx = 0;
            double sumx2 = 0;
            double sumxy = 0;
            double min = 10e20;
            double max = -10e20;
            int nbVals = 0;
            for(int j = 1; j < nbCols - m_nbVirtualColumns; ++j) {
                QString valstring = newLine.at(j);
                if(!valstring.isEmpty()) {
                    double v = SKGServices::stringToDouble(valstring);
                    sumx += j;
                    sumx2 += j * j;
                    sumy += v;
                    sumxy += j * v;
                    min = qMin(min, v);
                    max = qMax(max, v);
                    ++nbVals;
                }
            }
            if(m_additionalInformation & SUM) newLine.push_back(SKGServices::doubleToString(sumy));
            if(m_additionalInformation & AVERAGE) newLine.push_back(SKGServices::doubleToString(sumy / nbVals));
            if(m_additionalInformation & LIMITS) {
                newLine.push_back(SKGServices::doubleToString(min));
                newLine.push_back(SKGServices::doubleToString(max));
            }

            if(nbVals) {
                double s2x = sumx2 / nbVals - sumx * sumx / (nbVals * nbVals);
                double sxy = sumxy / nbVals - (sumx / nbVals) * (sumy / nbVals);

                double a = (s2x ? sxy / s2x : 0.0);
                double b = sumy / nbVals - a * sumx / nbVals;

                newLine.push_back("y=" + SKGServices::doubleToString(a) + "*x+" + SKGServices::doubleToString(b));
            } else newLine.push_back("y=0");

            groupedTable.replace(i, newLine);
        }
        ++nbCols;
        if(m_additionalInformation & SUM) ++nbCols;
        if(m_additionalInformation & AVERAGE) ++nbCols;
        if(m_additionalInformation & LIMITS) nbCols += 2;
    }

    //Sort lines
    if(m_sortColumn != 0  || m_sortOrder != Qt::AscendingOrder) {
        SKGTRACEIN(10, "SKGTableWithGraph::refresh-sort");
        //Extract title and sums
        if(groupedTable.count()) {
            QStringList fist = groupedTable.takeFirst();
            QStringList last;
            if(m_additionalInformation & SUM && groupedTable.count()) last = groupedTable.takeLast();

            //Sort
            qSort(groupedTable.begin(), groupedTable.end(), listSort);

            //Add title and sums
            groupedTable.insert(0, fist);
            if(m_additionalInformation & SUM) groupedTable.push_back(last);
        }
    }

    IFSKGTRACEL(10) {
        QStringList dump = SKGServices::tableToDump(groupedTable, SKGServices::DUMP_TEXT);
        int nbl = dump.count();
        for(int i = 0; i < nbl; ++i) {
            SKGTRACE << dump[i] << endl;
        }
    }

    //Fill table
    {
        SKGTRACEIN(10, "SKGTableWithGraph::refresh-fill");

        int nbRealColumns = nbRealCols - m_nbVirtualColumns;
        KLocale* locale = KGlobal::locale();
        ui.kTable->hide();
        QHeaderView* horizontalHeader = ui.kTable->horizontalHeader();
        if(horizontalHeader) horizontalHeader->setResizeMode(QHeaderView::Fixed);       //Needed to improve performances of setHorizontalHeaderLabels
        ui.kTable->clear();
        ui.kTable->setRowCount(nblines - 1);
        ui.kTable->setColumnCount(nbCols);
        for(int i = 0; i < nblines; ++i) {
            QStringList line = groupedTable.at(i);
            if(i == 0) {
                SKGTRACEIN(10, "SKGTableWithGraph::refresh-sethorizontalheaderlabels");
                //Set header
                ui.kTable->setHorizontalHeaderLabels(line);

                //Hide linear regression
                ui.kTable->hideColumn(m_indexLinearRegression);
            } else {
                for(int j = 0; j < nbCols; ++j) {
                    QString val = line.at(j);

                    QTableWidgetItem* item;
                    if(j == 0) {
                        //Create the line header
                        if(m_sumRows[i]) {
                            item = new QTableWidgetItem(val.isEmpty() ?
                                                        i18nc("Noun, the numerical sum of a list of values", "Sum")
                                                        : i18nc("Noun, the numerical sum of a list of values", "Sum of '%1'", val));
                            item->setData(1, val);
                            QFont f = item->font();
                            f.setBold(true);
                            item->setFont(f);
                            if(m_indexLinearRegression != -1) item->setData(DATA_VALUE, line.at(m_indexLinearRegression));
                            ui.kTable->setItem(i - 1, j, item);
                        } else {
                            //New color selector
                            QColor defaultColor;
                            int color_h = (240 + 360 * (i - 1) / nblines) % 360; //First color is blue to be compliant with min (red) and max (green)
                            defaultColor = QColor::fromHsv(color_h, 255, 255);

                            QColor color;
                            if(m_mapTitleColor.contains(val)) {
                                color = m_mapTitleColor[val];
                            } else {
                                color = defaultColor;
                                m_mapTitleColor[val] = color;
                            }

                            SKGColorButton* colorSelector = new SKGColorButton(this);
                            colorSelector->setText(val);
                            colorSelector->setColor(color);
                            colorSelector->setDefaultColor(defaultColor);
                            connect(colorSelector, SIGNAL(changed(QColor)), this, SLOT(onChangeColor()));

                            ui.kTable->setCellWidget(i - 1, 0, colorSelector);
                        }
                    } else {
                        //Add a value
                        if(!val.isEmpty()) {
                            if(j == m_indexLinearRegression) {
                                //A linear regression value
                                item = new QTableWidgetItem(val);
                            } else {
                                //A single value
                                double vald = SKGServices::stringToDouble(val);

                                item = new QTableWidgetItem(locale->formatMoney(vald, m_primaryUnit.Symbol, m_primaryUnit.NbDecimal));
                                if(!m_secondaryUnit.Symbol.isEmpty() && m_secondaryUnit.Value) {
                                    item->setToolTip(locale->formatMoney(vald / m_secondaryUnit.Value, m_secondaryUnit.Symbol, m_secondaryUnit.NbDecimal));
                                }

                                item->setData(DATA_VALUE, vald);
                                item->setTextAlignment(Qt::AlignRight);
                                if(vald < 0) item->setForeground(QBrush(QColor(Qt::red)));
                                if(m_sumRows[i]) {
                                    QFont f = item->font();
                                    f.setBold(true);
                                    item->setFont(f);
                                }
                            }
                        } else {
                            //An empty value
                            item = new QTableWidgetItem("");
                            item->setData(DATA_VALUE, "");
                        }
                        item->setFlags((j >= nbRealColumns && j != m_indexSum) || ui.kTable->horizontalHeaderItem(j)->text() == "0000" ? Qt::NoItemFlags : Qt::ItemIsEnabled | Qt::ItemIsSelectable);
                        ui.kTable->setItem(i - 1, j, item);
                    }
                }
            }
        }

        //Refresh graphic view
        redrawGraph();

        //Refresh text area
        redrawText();

        if(horizontalHeader) horizontalHeader->setResizeMode(QHeaderView::ResizeToContents);
        ui.kTable->show();
    }

    QApplication::restoreOverrideCursor();
}

void SKGTableWithGraph::onChangeColor()
{
    SKGColorButton* colorButton = qobject_cast<SKGColorButton*>(sender());
    if(colorButton) {
        m_mapTitleColor[colorButton->text()] = colorButton->color();
        refresh();
    }
}

void SKGTableWithGraph::onResetColors()
{
    m_mapTitleColor.clear();
    refresh();
}

void SKGTableWithGraph::onSelectionChanged()
{
    if(!m_graphVisible) return;
    _SKGTRACEIN(10, "SKGTableWithGraph::onSelectionChanged");

    //Unset color on previous selection
    int nbRow = ui.kTable->rowCount();
    int nbCol = ui.kTable->columnCount();
    for(int r = 0; r < nbRow; ++r) {
        for(int c = 0; c < nbCol; ++c) {
            QTableWidgetItem* previous = ui.kTable->item(r, c);
            if(previous) {
                QAbstractGraphicsShapeItem * graphicItem = dynamic_cast<QAbstractGraphicsShapeItem *>((QGraphicsItem*) previous->data(1).toLongLong());
                if(graphicItem) {
                    graphicItem->setBrush(QBrush(QColor::fromHsv(graphicItem->data(DATA_COLOR_H).toInt(),
                                                 graphicItem->data(DATA_COLOR_S).toInt(),
                                                 graphicItem->data(DATA_COLOR_V).toInt())));
                    if(graphicItem->isSelected()) graphicItem->setSelected(false);
                } else {
                    QGraphicsLineItem * graphicItem = dynamic_cast<QGraphicsLineItem *>((QGraphicsItem*) previous->data(1).toLongLong());
                    if(graphicItem) {
                        QPen pen = graphicItem->pen();
                        pen.setColor(QColor::fromHsv(graphicItem->data(DATA_COLOR_H).toInt(),
                                                     graphicItem->data(DATA_COLOR_S).toInt(),
                                                     graphicItem->data(DATA_COLOR_V).toInt()));
                        graphicItem->setPen(pen);
                    }
                }
            }
        }
    }

    //Set highlight color on current selection
    QList<QTableWidgetItem*> selected = ui.kTable->selectedItems();
    int nb = selected.count();
    for(int i = 0; i < nb; ++i) {
        QTableWidgetItem* current = selected.at(i);
        if(current) {
            QAbstractGraphicsShapeItem * graphicItem = dynamic_cast<QAbstractGraphicsShapeItem *>((QGraphicsItem*) current->data(1).toLongLong());
            if(graphicItem) {
                graphicItem->setBrush(QBrush(QApplication::palette().color(QPalette::Highlight)));
                if(!graphicItem->isSelected()) graphicItem->setSelected(true);
            } else {
                QGraphicsLineItem * graphicItem = dynamic_cast<QGraphicsLineItem *>((QGraphicsItem*) current->data(1).toLongLong());
                if(graphicItem) {
                    QPen pen = graphicItem->pen();
                    pen.setColor(QApplication::palette().color(QPalette::Highlight));
                    graphicItem->setPen(pen);
                }
            }
        }
    }
}

void SKGTableWithGraph::onDoubleClickGraph()
{
    if(m_scene) {
        //Get selection
        QList<QGraphicsItem *> selectedGraphItems = m_scene->selectedItems();
        if(selectedGraphItems.count()) {
            Q_EMIT cellDoubleClicked(selectedGraphItems[0]->data(1).toInt(), selectedGraphItems[0]->data(2).toInt());
        }
    }
}

void SKGTableWithGraph::onDoubleClick(int row, int column)
{
    Q_EMIT cellDoubleClicked(row, column);
}

void SKGTableWithGraph::onSelectionChangedInGraph()
{
    _SKGTRACEIN(10, "SKGTableWithGraph::onSelectionChangedInGraph");
    if(m_scene) {
        bool previous = ui.kTable->blockSignals(true);
        ui.kTable->clearSelection();

        //Get selection
        QList<QGraphicsItem *> selectedGraphItems = m_scene->selectedItems();
        int nb = selectedGraphItems.count();
        for(int i = 0; i < nb; ++i) {
            ui.kTable->setCurrentCell(selectedGraphItems[i]->data(1).toInt(), selectedGraphItems[i]->data(2).toInt(), QItemSelectionModel::Select);
        }
        ui.kTable->blockSignals(previous);

        previous = m_scene->blockSignals(true);
        onSelectionChanged();
        m_scene->blockSignals(previous);
    }
}

void SKGTableWithGraph::redrawGraphDelayed()
{
    m_timerRedraw.start(300);
}

void SKGTableWithGraph::redrawText()
{
    SKGTRACEIN(10, "SKGTableWithGraph::redrawText");
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

    //Set style sheet
    QDir dirCss(KStandardDirs::locate("data", QString::fromLatin1("skrooge/css/")));
    QFile file(dirCss.absoluteFilePath("skrooge_monthly-default.css"));
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QString style;
    QTextStream in(&file);
    while(!in.atEnd()) {
        style += in.readLine();
    }

    QString html = "<? xml version = \"1.0\" encoding=\"utf-8\"?>"
                   "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
                   "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
                   "<head>"
                   "<meta http-equiv=\"content-type\" content=\"text/html;charset=utf-8\" />"
                   "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\" />"
                   "<style type=\"text/css\">" + style + "</style>"
                   "</head>"
                   "<body>"
                   "<table class=\"table\"><tr class=\"tabletitle\">";
    //Dump header
    int nbCols = ui.kTable->columnCount();
    for(int i = 0; i < nbCols; ++i) {
        QTableWidgetItem* item = ui.kTable->horizontalHeaderItem(i);
        if(item) html += "<td align=\"center\"><b>" + item->text() + "</b></td>";
    }
    html += "</tr>";

    //Dump values
    int nbLines = ui.kTable->rowCount();
    for(int j = 0; j < nbLines; ++j) {
        html += QString("<tr") + (m_sumRows[j+1] ? " class=\"tabletotal\"" : "") + '>';
        for(int i = 0; i < nbCols; ++i) {
            QTableWidgetItem* item = ui.kTable->item(j, i);
            if(item) {
                bool red = (item->data(DATA_VALUE).toDouble() < 0);
                html += QString("<td align=\"right\">") + (red ? "<font color=\"red\">" : "") + item->text() + (red ? "</font>" : "") + "</td>";
            } else {
                SKGColorButton* item = qobject_cast<SKGColorButton*>(ui.kTable->cellWidget(j, i));
                if(item) html += "<td>" + item->text() + "</td>";
            }
        }
        html += "</tr>";
    }
    html += "</table>";
    html += "</body></html>";
    ui.kTextEdit->setHtml(html);
    QApplication::restoreOverrideCursor();
}

void SKGTableWithGraph::redrawGraph()
{
    SKGTRACEIN(10, "SKGTableWithGraph::redrawGraph");
    if(!m_graphVisible) return;
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

    ui.graphicView->hide();
    ui.kTable->hide();

    //Recreate scene
    if(m_scene) {
        SKGTRACEIN(10, "SKGTableWithGraph::redrawGraph-remove scene");
        m_scene->clear();
        delete m_scene;
    }

    m_scene = new SKGGraphicsScene();
    {
        SKGTRACEIN(10, "SKGTableWithGraph::redrawGraph-create scene");
        //Get current selection
        int crow = ui.kTable->currentRow();
        int ccolumn = ui.kTable->currentColumn();

        //Get nb columns and rows
        int nbRows = ui.kTable->rowCount();
        int nbRealRows = nbRows;
        for(int y = 0; y < nbRows; ++y) {
            if(m_sumRows[y+1]) --nbRealRows;
        }

        int nbColumns = getNbColumns(false);
        int nbRealColumns = nbColumns - m_nbVirtualColumns;

        //Get graphic mode
        GraphType mode = (GraphType) ui.kDisplayMode->currentIndex();
        bool inPositive = false;
        if(mode == STACK || mode == HISTOGRAM || mode == POINT || mode == LINE) {
            ui.kAllPositive->show();
            inPositive = (ui.kAllPositive->checkState() == Qt::Checked);
        } else ui.kAllPositive->hide();

        //Compute y limits
        double minLimit = 0;
        double maxLimit = 0;
        int nbLevel = 0;
        SKGTRACEL(3) << "mode=" << mode << endl;
        SKGTRACEL(3) << "nb rows        =" << nbRows << endl;
        SKGTRACEL(3) << "nb real rows   =" << nbRealRows << endl;
        SKGTRACEL(3) << "nb columns     =" << nbColumns << endl;
        SKGTRACEL(3) << "nb real columns=" << nbRealColumns << endl;
        SKGTRACEL(3) << "selected row   =" << crow << endl;
        SKGTRACEL(3) << "selected column=" << ccolumn << endl;
        if(mode == STACK) {
            //STACK
            for(int x = 0; x < nbRows; ++x) {
                if(!m_sumRows[x+1]) {
                    double sumPositive = 0;
                    double sumNegative = 0;
                    for(int y = 0; y < nbColumns; ++y) {
                        QTableWidgetItem* tableItem = ui.kTable->item(x, y);
                        if(tableItem) {
                            double val = tableItem->data(DATA_VALUE).toDouble();
                            if(inPositive || val > 0) {
                                sumPositive += fabs(val);
                            } else {
                                sumNegative += val;
                            }
                        }
                    }

                    minLimit = qMin(minLimit, sumNegative);
                    maxLimit = qMax(maxLimit, sumPositive);
                }
            }
        } else if(mode == HISTOGRAM || mode == POINT || mode == LINE) {
            //HISTOGRAM or POINTS or LINES
            for(int x = 0; x < nbRows; ++x) {
                if(!m_sumRows[x+1]) {
                    for(int y = 0; y < nbColumns; ++y) {
                        QTableWidgetItem* tableItem = ui.kTable->item(x, y);
                        if(tableItem) {
                            double val = tableItem->data(DATA_VALUE).toDouble();
                            if(inPositive) {
                                maxLimit = qMax(maxLimit, fabs(val));
                            } else {
                                minLimit = qMin(minLimit, val);
                                maxLimit = qMax(maxLimit, val);
                            }
                        }
                    }
                }
            }
        } else if(mode == CONCENTRICPIE) {
            for(int x = 0; x < nbRows; ++x) {
                SKGColorButton* btn = qobject_cast<SKGColorButton*>(ui.kTable->cellWidget(x, 0));
                if(btn) {
                    QString xname = btn->text();
                    QStringList vals = xname.split(OBJECTSEPARATOR);
                    int nbvals = vals.count();
                    nbLevel = qMax(nbLevel, nbvals - 1);
                }
            }
        }
        SKGTRACEL(30) << "minLimit=" << minLimit << endl;
        SKGTRACEL(30) << "maxLimit=" << maxLimit << endl;

        //Compute
        double width = 10;
        double maxX = 0;
        double margin = 0;
        if(nbRealRows) {
            QRect vSize = ui.graphicView->rect();
            if(mode == STACK) {
                width = (maxLimit - minLimit) / nbRealRows;
                width = width * ((double) vSize.width()) / ((double) vSize.height());
                margin = (maxLimit - minLimit) * 0.5;
                maxX = width * nbRealRows + margin;
            } else if(mode == HISTOGRAM || mode == POINT || mode == LINE) {
                width = (maxLimit - minLimit) / (nbRealRows * (nbColumns - 1));
                width = width * ((double) vSize.width()) / ((double) vSize.height());
                margin = (maxLimit - minLimit) * 0.3;
                maxX = width * (nbColumns - 1) * (nbRealRows + 1) ;
            }
        }

        SKGTRACEL(30) << "width=" << width << endl;
        SKGTRACEL(30) << "maxX=" << maxX << endl;
        SKGTRACEL(30) << "margin=" << margin << endl;

        //Initialise pens
        QPen axisPen = QPen(Qt::SolidLine);
        axisPen.setColor(Qt::gray);
        axisPen.setWidth(margin / 30);

        QPen gridPen = QPen(Qt::SolidLine);
        gridPen.setColor(Qt::lightGray);
        gridPen.setWidth(margin / 100);

        QPen dotPen = QPen(Qt::SolidLine);  //WARNING: Qt::DotLine is very bad for performances
        dotPen.setColor(Qt::gray);
        dotPen.setWidth(margin / 50);

        QPen blackPen = QPen(Qt::SolidLine);  //WARNING: Qt::DotLine is very bad for performances
        blackPen.setColor(Qt::black);
        blackPen.setWidth(margin / 100);

        //Set rect
        int nbColInMode2 = (nbColumns > 2 ? sqrt(nbColumns - 1) + .5 : 1);
        m_scene->setItemIndexMethod(QGraphicsScene::NoIndex);
        SKGTRACEL(10) << "Items:" << nbRealRows << "x" << (nbColumns - 1) << "=" << nbRealRows*(nbColumns - 1) << endl;

        //Redraw scene
        double x0 = 0;
        double x1 = 0;
        double ymin = 0;
        double ymax = 0;
        int x = 0;
        for(int xx = 0; xx < nbRows; ++xx) {
            SKGColorButton* colorButton = qobject_cast<SKGColorButton*>(ui.kTable->cellWidget(xx, 0));
            if(colorButton) {
                QString xname = colorButton->text();
                QColor initialColor = colorButton->color();
                double yPlus = 0;
                double yMoins = 0;
                int jprevious = -1;
                for(int j = 1; j < nbColumns; ++j) {
                    //Get column name
                    QString yname = ui.kTable->horizontalHeaderItem(j)->text();

                    //Get cell value
                    QTableWidgetItem* tableItem = ui.kTable->item(xx, j);
                    if(tableItem) {
                        double val = tableItem->data(DATA_VALUE).toDouble();
                        QString valstring = tableItem->text();

                        int color_h;
                        int color_s;
                        int color_v;
                        initialColor.getHsv(&color_h, &color_s, &color_v);
                        color_s = color_s - color_s * 155 / 255 * (nbColumns - 1 - j) / nbColumns;
                        color_v = color_v - color_v * 155 / 255 * (nbColumns - 1 - j) / nbColumns;
                        if(j >= nbRealColumns) {
                            //Yellow
                            color_h = 60;
                            color_s = 255;
                            color_v = 255;
                        }

                        QColor color;
                        if(x == crow && j == ccolumn) color = QApplication::palette().color(QPalette::Highlight);
                        else color = QColor::fromHsv(color_h, color_s, color_v);

                        QBrush brush(color);

                        QGraphicsItem* graphItem = NULL;
                        if(mode == STACK) {
                            //STACK
                            if(val > 0 || inPositive) {
                                graphItem = m_scene->addRect(width * x, -yPlus - fabs(val), width, fabs(val), QPen(), brush);
                                yPlus += fabs(val);
                                if(yPlus > ymax) ymax = yPlus;
                            } else {
                                graphItem = m_scene->addRect(width * x, -yMoins, width, -val, QPen(), brush);
                                yMoins += val;
                                if(yMoins < ymin) ymin = yMoins;
                            }
                        } else if(mode == HISTOGRAM) {
                            //HISTOGRAM
                            if(j == 1) x0 = width * ((j - 1) * (nbRealRows + 1) + x) + width;
                            if(j == nbColumns - m_nbVirtualColumns - 1) x1 = width * ((j - 1) * (nbRealRows + 1) + x) + width;
                            if(inPositive) {
                                graphItem = m_scene->addRect(width * ((j - 1) * (nbRealRows + 1) + x) + width / 2, -fabs(val), width, fabs(val), QPen(), brush);
                                if(fabs(val) > ymax) ymax = fabs(val);
                            } else {
                                graphItem = m_scene->addRect(width * ((j - 1) * (nbRealRows + 1) + x) + width / 2, (val < 0 ? 0 : -val), width, (val < 0 ? -val : val), QPen(), brush);
                                if(val > ymax) ymax = val;
                                if(val < ymin) ymin = val;
                            }
                        } else if(mode == POINT) {
                            //POINTS
                            double radius = margin / 8;
                            double xmin = width * (j - 1) * (nbRealRows + 1) - radius;
                            if(j == 1) x0 = xmin + radius;
                            if(j == nbColumns - m_nbVirtualColumns - 1) x1 = xmin + radius;

                            if(inPositive) {
                                switch(x % 3) {
                                case 0:
                                    graphItem = m_scene->addEllipse(xmin, -fabs(val) - radius, 2 * radius, 2 * radius, QPen(), brush);
                                    break;
                                case 1:
                                    graphItem = m_scene->addRect(xmin, -fabs(val) - radius, 2 * radius, 2 * radius, QPen(), brush);
                                    break;
                                case 2:
                                    QPolygonF polygon;
                                    polygon << QPointF(xmin, -fabs(val) - radius) << QPointF(xmin + 2 * radius, -fabs(val) + radius)
                                            << QPointF(xmin + 2 * radius, -fabs(val) - radius) << QPointF(xmin, -fabs(val) + radius);
                                    graphItem = m_scene->addPolygon(polygon, QPen(), brush);
                                    break;
                                }
                                if(fabs(val) > ymax) ymax = fabs(val);
                            } else {
                                switch(x % 3) {
                                case 0:
                                    graphItem = m_scene->addEllipse(xmin, -val - radius, 2 * radius, 2 * radius, QPen(), brush);
                                    break;
                                case 1:
                                    graphItem = m_scene->addRect(xmin, -val - radius, 2 * radius, 2 * radius, QPen(), brush);
                                    break;
                                case 2:
                                    QPolygonF polygon;
                                    polygon << QPointF(xmin, -val - radius) << QPointF(xmin + 2 * radius, -val + radius)
                                            << QPointF(xmin + 2 * radius, -val - radius) << QPointF(xmin, -val + radius);
                                    graphItem = m_scene->addPolygon(polygon, QPen(), brush);
                                    break;
                                }
                                if(val > ymax) ymax = val;
                                if(val < ymin) ymin = val;
                            }

                        } else if(mode == LINE) {
                            //LINES
                            //Empty cells are ignored
                            if(j == 1) x0 = width * (j - 1) * (nbRealRows + 1);
                            if(j == nbColumns - m_nbVirtualColumns - 1) x1 = width * (j - 1) * (nbRealRows + 1);

                            if(!valstring.isEmpty()) {
                                if(jprevious == -1) jprevious = j;

                                //Create pen
                                QPen pen = QPen(color);
                                pen.setWidth(margin / 10);
                                pen.setCapStyle(Qt::RoundCap);
                                pen.setJoinStyle(Qt::RoundJoin);

                                QTableWidgetItem* tableItem2 = ui.kTable->item(xx, jprevious);
                                if(tableItem2) {
                                    QString val2string = tableItem->text();
                                    if(!val2string.isEmpty()) {
                                        double val2 = tableItem2->data(DATA_VALUE).toDouble();

                                        if(inPositive) {
                                            graphItem = m_scene->addLine(width * (jprevious - 1) * (nbRealRows + 1) - (jprevious == j ? width / 20 : 0), -fabs(val2), width * (j - 1) * (nbRealRows + 1), -fabs(val), pen);
                                            if(fabs(val) > ymax) ymax = fabs(val);
                                        } else {
                                            graphItem = m_scene->addLine(width * (jprevious - 1) * (nbRealRows + 1) - (jprevious == j ? width / 20 : 0), -val2, width * (j - 1) * (nbRealRows + 1), -val, pen);
                                            if(val > ymax) ymax = val;
                                            if(val < ymin) ymin = val;
                                        }
                                        jprevious = j;
                                    }
                                }
                            }
                        } else if(mode == PIE || mode == CONCENTRICPIE) {
                            //PIE
                            //Compute absolute sum of the column
                            val = fabs(val);
                            double sumColumn = 0;
                            double previousSum = 0;
                            for(int x2 = 0; x2 < nbRows; ++x2) {
                                if(!m_sumRows[x2+1]) {
                                    QTableWidgetItem* tableItem = ui.kTable->item(x2, j);
                                    if(tableItem) {
                                        double absVal = fabs(tableItem->data(DATA_VALUE).toDouble());
                                        sumColumn += absVal;
                                        if(x2 < xx) previousSum += absVal;
                                    }
                                }
                            }

                            if(sumColumn) {
                                int nbvals = xname.split(OBJECTSEPARATOR).count();
                                double step = 0;
                                double p = 0;
                                if(mode == CONCENTRICPIE) {
                                    step = 100 / (nbLevel + 1);
                                    p = step * (nbLevel + 1 - nbvals);
                                }

                                QPainterPath path;
                                path.moveTo(BOX_SIZE*((j - 1) % nbColInMode2) + BOX_SIZE / 2, BOX_SIZE * floor((j - 1) / nbColInMode2) + BOX_SIZE / 2);
                                path.arcTo(BOX_SIZE*((j - 1) % nbColInMode2) + BOX_MARGIN + p / 2, BOX_SIZE * floor((j - 1) / nbColInMode2) + BOX_MARGIN + p / 2, BOX_SIZE - 2 * BOX_MARGIN - p, BOX_SIZE - 2 * BOX_MARGIN - p, 360 * previousSum / sumColumn, 360 * val / sumColumn);
                                if(mode == CONCENTRICPIE && nbvals <= nbLevel + 1 && nbvals > 1) {
                                    p = step * (nbLevel + 1 - nbvals + 1);
                                    path.moveTo(BOX_SIZE*((j - 1) % nbColInMode2) + BOX_SIZE / 2, BOX_SIZE * floor((j - 1) / nbColInMode2) + BOX_SIZE / 2);
                                    path.arcTo(BOX_SIZE*((j - 1) % nbColInMode2) + BOX_MARGIN + p / 2, BOX_SIZE * floor((j - 1) / nbColInMode2) + BOX_MARGIN + p / 2, BOX_SIZE - 2 * BOX_MARGIN - p, BOX_SIZE - 2 * BOX_MARGIN - p, 360 * previousSum / sumColumn, 360 * val / sumColumn);
                                }
                                path.closeSubpath();
                                graphItem = m_scene->addPath(path, QPen(Qt::white), brush);
                            }
                        }

                        if(graphItem) {
                            graphItem->setZValue(5);
                            graphItem->setToolTip(xname + '\n' + yname + '\n' + valstring + '\n' + tableItem->toolTip());
                            bool isSelect = (isSelectable() && tableItem->flags() & Qt::ItemIsEnabled);
                            graphItem->setFlag(QGraphicsItem::ItemIsSelectable, isSelect);
                            if(isSelect) graphItem->setCursor(Qt::PointingHandCursor);
                            graphItem->setData(1, xx);
                            graphItem->setData(2, j);
                            graphItem->setData(DATA_COLOR_H, color_h);
                            graphItem->setData(DATA_COLOR_S, color_s);
                            graphItem->setData(DATA_COLOR_V, color_v);

                            tableItem->setData(1, (qlonglong)  graphItem);
                        }
                    } else {
                        SKGTRACE << "WARNING: cell " << x << "," << j << " null" << endl;
                    }
                }

                ++x;
            }
        }

        //Draw axis
        double scaleText = 0;
        int nbRowInMode2 = ((int)((nbColumns - 1) / nbColInMode2));
        if(nbRealRows) {
            if(mode == PIE || mode == CONCENTRICPIE) {
                //PIE
                if(nbRowInMode2 * nbColInMode2 < nbColumns - 1) ++nbRowInMode2;
                for(int i = 0; i <= nbColInMode2; ++i) {
                    QGraphicsLineItem* item = m_scene->addLine(BOX_SIZE * i, 0, BOX_SIZE * i, BOX_SIZE * nbRowInMode2);
                    item->setPen(axisPen);
                    item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                }
                for(int i = 0; i <= nbRowInMode2; ++i) {
                    QGraphicsLineItem* item = m_scene->addLine(0, BOX_SIZE * i, BOX_SIZE * nbColInMode2, BOX_SIZE * i);
                    item->setPen(axisPen);
                    item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                }

                for(int j = 1; j < nbColumns; ++j) {
                    //Get column name
                    QString yname = ui.kTable->horizontalHeaderItem(j)->text();

                    QGraphicsTextItem* textItem = m_scene->addText(yname);
                    textItem->setPos(BOX_SIZE*((j - 1) % nbColInMode2) + 2, BOX_SIZE * floor((j - 1) / nbColInMode2) + 2);
                    textItem->scale(.5, .5);
                    textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);

                }
            } else {
                //STACK & HISTOGRAMM
                QGraphicsLineItem* item;

                //Lines x
                double ystep = computeStepSize(ymax - ymin, 10);
                if(ystep > 0) {
                    KLocale* locale = KGlobal::locale();
                    for(double y = ystep; y < ymax + margin; y = y + ystep) {
                        item = m_scene->addLine(-margin, -y, maxX, -y);
                        item->setPen(gridPen);
                        item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                        item->setZValue(1);

                        QGraphicsTextItem* textItem = m_scene->addText(locale->formatMoney(y, m_primaryUnit.Symbol, m_primaryUnit.NbDecimal));
                        textItem->setPos(-margin, -y);
                        if(scaleText == 0) {
                            QRectF rect = textItem->boundingRect();
                            scaleText = 0.8 * qMin(margin / rect.width(), ystep / rect.height());
                        }
                        textItem->scale(scaleText, scaleText);
                        textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);
                    }
                    for(double y = -ystep; y > ymin - margin; y = y - ystep) {
                        item = m_scene->addLine(-margin, -y, maxX, -y);
                        item->setPen(gridPen);
                        item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                        item->setZValue(1);

                        QGraphicsTextItem* textItem = m_scene->addText(locale->formatMoney(y, m_primaryUnit.Symbol, m_primaryUnit.NbDecimal));
                        textItem->setPos(-margin, -y);
                        if(scaleText == 0) {
                            QRectF rect = textItem->boundingRect();
                            scaleText = 0.8 * qMin(margin / rect.width(), ystep / rect.height());
                        }
                        textItem->scale(scaleText, scaleText);
                        textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);
                    }
                }

                if(mode == HISTOGRAM || mode == POINT || mode == LINE) {
                    //Line y
                    double xstep = width * (nbRealRows + 1);
                    int jstep = qMax(computeStepSize(nbColumns, 20), double(1.0));
                    for(int j = 1; j < nbColumns; j += jstep) {
                        QString yname = ui.kTable->horizontalHeaderItem(j)->text();
                        double x = xstep + (j - 2) * xstep;

                        QGraphicsTextItem* textItem = m_scene->addText(yname);
                        textItem->setPos(x, 0);
                        textItem->scale(scaleText, scaleText);
                        textItem->rotate(90);
                        textItem->moveBy(textItem->boundingRect().height() *scaleText, 0);
                        textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);

                        QGraphicsLineItem* item = m_scene->addLine(x, -ymin + margin, x, -ymax - margin);
                        item->setPen(gridPen);
                        item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                    }
                }

                //Axis y
                item = m_scene->addLine(0, -ymin + margin, 0, -ymax - margin, axisPen);
                item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                addArrow(QPointF(0, -ymax - margin - margin / 6), margin / 6, 45, 90);
                item->setZValue(2);

                //Axis x
                item = m_scene->addLine(-margin, 0, maxX, 0, axisPen);
                item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                addArrow(QPointF(maxX + margin / 6, 0), margin / 6, 45, 180);
                item->setZValue(2);
            }
        }

        //Draw Average
        bool lineCondition = m_limitVisible && (mode == HISTOGRAM || mode == POINT || mode == LINE) && scaleText && nbRows == 1;
        int averageCol = getAverageColumnIndex();
        if(lineCondition && averageCol != -1) {
            QTableWidgetItem* tableItem = ui.kTable->item(0, averageCol);
            if(tableItem) {
                double y = tableItem->data(DATA_VALUE).toDouble();
                if(inPositive) y = fabs(y);
                KLocale* locale = KGlobal::locale();
                QGraphicsLineItem* item = m_scene->addLine(0, -y, maxX, -y);
                dotPen.setColor(Qt::blue);
                item->setPen(dotPen);
                item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                item->setZValue(1);

                QGraphicsTextItem* textItem = m_scene->addText(locale->formatMoney(y, m_primaryUnit.Symbol, m_primaryUnit.NbDecimal));
                textItem->setPos(maxX, -y);
                textItem->setDefaultTextColor(Qt::blue);
                textItem->scale(scaleText, scaleText);
                textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);
            }
        }

        //Draw Min & Max limites
        int minCol = getMinColumnIndex();
        if(lineCondition && minCol != -1) {
            //Min
            {
                QTableWidgetItem* tableItem = ui.kTable->item(0, minCol);
                if(tableItem) {
                    double y = tableItem->data(DATA_VALUE).toDouble();
                    if(inPositive) y = fabs(y);
                    KLocale* locale = KGlobal::locale();
                    QGraphicsLineItem* item = m_scene->addLine(0, -y, maxX, -y);
                    dotPen.setColor(Qt::red);
                    item->setPen(dotPen);
                    item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                    item->setZValue(1);

                    QGraphicsTextItem* textItem = m_scene->addText(locale->formatMoney(y, m_primaryUnit.Symbol, m_primaryUnit.NbDecimal));
                    textItem->setPos(maxX, -y);
                    textItem->setDefaultTextColor(Qt::red);
                    textItem->scale(scaleText, scaleText);
                    textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);
                }
            }

            //Max
            {
                QTableWidgetItem* tableItem = ui.kTable->item(0, minCol + 1);
                if(tableItem) {
                    double y = tableItem->data(DATA_VALUE).toDouble();
                    if(inPositive) y = fabs(y);
                    KLocale* locale = KGlobal::locale();
                    QGraphicsLineItem* item = m_scene->addLine(0, -y, maxX, -y);
                    dotPen.setColor(Qt::green);
                    item->setPen(dotPen);
                    item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                    item->setZValue(1);

                    QGraphicsTextItem* textItem = m_scene->addText(locale->formatMoney(y, m_primaryUnit.Symbol, m_primaryUnit.NbDecimal));
                    textItem->setPos(maxX, -y);
                    textItem->setDefaultTextColor(Qt::green);
                    textItem->scale(scaleText, scaleText);
                    textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);
                }
            }
        }

        //Draw Linear Regression
        bool lineCondition2 = m_linearRegressionVisible && (mode == HISTOGRAM || mode == POINT || mode == LINE) && scaleText && nbRows == 1;
        if(lineCondition2 && m_indexLinearRegression != -1) {
            QTableWidgetItem* tableItem = ui.kTable->item(0, m_indexLinearRegression);
            if(tableItem) {
                QString f = tableItem->text();

                QScriptEngine myEngine;
                QString f0 = f;
                f0 = f0.remove("y=");
                f0 = f0.replace('x', '1');
                f0 = f0.replace(',', '.'); // Replace comma by a point in case of typo
                if(inPositive) f0 = "Math.abs(" + f0 + ')';
                double y0 = myEngine.evaluate(f0).toNumber();

                QString f1 = f;
                f1 = f1.remove("y=");
                f1 = f1.replace('x', SKGServices::intToString(nbRealColumns - 1));
                f1 = f1.replace(',', '.'); // Replace comma by a point in case of typo
                if(inPositive) f1 = "Math.abs(" + f1 + ')';
                double y1 = myEngine.evaluate(f1).toNumber();

                QGraphicsLineItem* item = m_scene->addLine(x0, -y0, x1, -y1);
                dotPen.setColor(Qt::darkYellow);
                item->setPen(dotPen);
                item->setFlag(QGraphicsItem::ItemIsSelectable, false);
                item->setZValue(1);

                /*QGraphicsTextItem* textItem=scene->addText ( f );
                textItem->setPos ( maxX, -y1 );
                textItem->setDefaultTextColor ( Qt::darkYellow );
                textItem->scale ( scaleText, scaleText );
                textItem->setFlag ( QGraphicsItem::ItemIsSelectable, false );*/
            }
        }

        //Draw legend
        if(m_legendVisible) {
            QPointF legendPosition;
            double maxY;
            if(mode == PIE || mode == CONCENTRICPIE) {
                int nbRowInMode2 = ((int)((nbColumns - 1) / nbColInMode2));
                if(nbRowInMode2 * nbColInMode2 < nbColumns - 1) ++nbRowInMode2;

                scaleText = 0.2;
                margin = BOX_SIZE / 10;
                legendPosition = QPointF(BOX_SIZE * nbColInMode2 + margin, 0);
                maxY = BOX_SIZE * nbRowInMode2;
            } else {
                legendPosition = QPointF(maxX + margin, -ymax - margin - margin / 6);
                maxY = -ymin + margin;
            }
            addLegend(legendPosition , margin / 4, scaleText, maxY);
        }
    }
    {
        SKGTRACEIN(10, "SKGTableWithGraph::redrawGraph-add scene to view");
        m_scene->setSceneRect(QRectF());
        ui.graphicView->setScene(m_scene);
        ui.graphicView->show();
        ui.kTable->show();
        ui.graphicView->onZoomOriginal();

        //Add selection event on scene
        connect(m_scene, SIGNAL(selectionChanged()), this, SLOT(onSelectionChangedInGraph()), Qt::QueuedConnection);
        connect(m_scene, SIGNAL(doubleClicked()), this, SLOT(onDoubleClickGraph()), Qt::QueuedConnection);
    }
    QApplication::restoreOverrideCursor();
}

int SKGTableWithGraph::getNbColumns(bool iWithComputed) const
{
    int nbColumns = ui.kTable->columnCount();
    if(!iWithComputed) {
        if(m_indexMin != -1) nbColumns -= 2;
        if(m_indexAverage != -1) --nbColumns;
        if(m_indexSum != -1) --nbColumns;
        if(m_indexLinearRegression != -1) --nbColumns;
    }
    return nbColumns;
}

int SKGTableWithGraph::getAverageColumnIndex() const
{
    return m_indexAverage;
}

int SKGTableWithGraph::getMinColumnIndex() const
{
    return m_indexMin;
}

double SKGTableWithGraph::computeStepSize(double iRange, double iTargetSteps)
{
    // Calculate an initial guess at step size
    double tempStep = iRange / iTargetSteps;
    // Get the magnitude of the step size
    double mag = floor(log10(tempStep));
    double magPow = pow((double) 10.0, mag);
    // Calculate most significant digit of the new step size
    double magMsd = ((int)(tempStep / magPow + .5));
    // promote the MSD to either 1, 2, or 5
    if(magMsd > 5.0) magMsd = 10.0;
    else if(magMsd > 2.0) magMsd = 5.0;
    else if(magMsd > 1.0) magMsd = 2.0;
    return magMsd * magPow;
}

void SKGTableWithGraph::addLegend(const QPointF& iPosition, double iSize, double iScaleText, double iMaxY)
{
    SKGTRACEIN(10, "SKGTableWithGraph::addLegend");

    if(m_scene) {
        int nbRows = ui.kTable->rowCount();
        int nbRealRows = nbRows;
        for(int y = 0; y < nbRows; ++y) {
            if(m_sumRows[y+1]) --nbRealRows;
        }
        int nbColumns = getNbColumns(false);
        if(nbColumns > 1) {
            double margin = 1.2;
            double currentYPos = iPosition.y();
            double currentXPos = iPosition.x();
            double currentXMaxSize = 0;
            for(int i = 0; i < nbRows; ++i) {
                SKGColorButton* btn = qobject_cast<SKGColorButton*>(ui.kTable->cellWidget(i, 0));
                if(btn) {
                    //Get title
                    QString title = btn->text();

                    //Build brush
                    QColor color1;
                    QTableWidgetItem* it = ui.kTable->item(i, 1);
                    if(it) {
                        QGraphicsItem * graphicItem = dynamic_cast<QGraphicsItem *>((QGraphicsItem*) it->data(1).toLongLong());
                        if(graphicItem) {
                            //Draw box
                            color1 = QColor::fromHsv(graphicItem->data(DATA_COLOR_H).toInt(),
                                                     graphicItem->data(DATA_COLOR_S).toInt(),
                                                     graphicItem->data(DATA_COLOR_V).toInt());
                        }
                    }
                    QColor color2;
                    it = ui.kTable->item(i, nbColumns - 1 - m_nbVirtualColumns);
                    if(it) {
                        QGraphicsItem * graphicItem = dynamic_cast<QGraphicsItem *>((QGraphicsItem*) it->data(1).toLongLong());
                        if(graphicItem) {
                            //Draw box
                            color2 = QColor::fromHsv(graphicItem->data(DATA_COLOR_H).toInt(),
                                                     graphicItem->data(DATA_COLOR_S).toInt(),
                                                     graphicItem->data(DATA_COLOR_V).toInt());
                        }
                    }

                    QLinearGradient grandient(currentXPos, currentYPos, currentXPos + 2 * iSize, currentYPos);
                    grandient.setColorAt(0, color1);
                    grandient.setColorAt(1, color2);

                    //Draw legend item
                    QGraphicsRectItem* item = m_scene->addRect(currentXPos, currentYPos , 2 * iSize, iSize, QPen(), QBrush(grandient));
                    item->setFlag(QGraphicsItem::ItemIsSelectable, false);

                    //Draw text
                    QGraphicsTextItem* textItem = m_scene->addText(title);
                    textItem->setPos(currentXPos + iSize + margin * iSize, currentYPos);
                    textItem->scale(iScaleText, iScaleText);
                    textItem->setFlag(QGraphicsItem::ItemIsSelectable, false);

                    //Compute next position
                    currentYPos += margin * iSize;
                    QRectF rect = textItem->boundingRect();
                    currentXMaxSize = qMax(currentXMaxSize, (double)(iScaleText * rect.width()));
                    if(currentYPos > iMaxY) {
                        currentYPos = iPosition.y();
                        currentXPos += currentXMaxSize + iSize + 2 * margin * iSize;
                        currentXMaxSize = 0;
                    }
                }
            }
        }
    }
}

void SKGTableWithGraph::addArrow(const QPointF& iPeak, double iSize, double iArrowAngle, double iDegree)
{
    if(m_scene) {
        QPolygonF pol;
        double radian = 3.14 * iArrowAngle / 360;
        pol << QPointF(0, 0) << QPointF(iSize * cos(radian), iSize * sin(radian)) << QPointF(iSize * cos(radian), -iSize * sin(radian)) << QPointF(0, 0);
        QGraphicsPolygonItem * item = m_scene->addPolygon(pol, QPen(Qt::black, iSize / 20), QBrush(Qt::black));
        item->rotate(iDegree);
        item->moveBy(iPeak.x(), iPeak.y());
        item->setFlag(QGraphicsItem::ItemIsSelectable, false);
        item->setZValue(2);
    }
}

void SKGTableWithGraph::onExport()
{
    _SKGTRACEIN(10, "SKGTableWithGraph::onExport");
    QString fileName = SKGMainPanel::getSaveFileName(KUrl("kfiledialog:///IMPEXP"), "text/csv text/plain" , this);
    if(fileName.isEmpty()) return;

    SKGError err;

    QString extension = QFileInfo(fileName).suffix().toUpper();
    if(extension == "CSV") {
        //Write file
        KSaveFile file(fileName);
        if(!file.open()) {
            err.setReturnCode(ERR_INVALIDARG);
            err.setMessage(i18nc("Error message",  "Save file '%1' failed", fileName));
        } else {
            QTextStream out(&file);
            QStringList dump = SKGServices::tableToDump(getTable(), SKGServices::DUMP_CSV);
            int nbl = dump.count();
            for(int i = 0; i < nbl; ++i) {
                out << dump[i] << endl;
            }
        }

        //Close file
        file.finalize();
        file.close();
    } else {
        SKGError err;

        //Write file
        KSaveFile file(fileName);
        if(!file.open()) {
            err.setReturnCode(ERR_INVALIDARG);
            err.setMessage(i18nc("Error message",  "Save file '%1' failed", fileName));
        } else {
            QTextStream out(&file);
            QStringList dump = SKGServices::tableToDump(getTable(), SKGServices::DUMP_TEXT);
            int nbl = dump.count();
            for(int i = 0; i < nbl; ++i) {
                out << dump[i] << endl;
            }
        }

        //Close file
        file.finalize();
        file.close();
    }
    SKGMainPanel::getMainPanel()->displayErrorMessage(err);
    QDesktopServices::openUrl(QUrl(fileName));
}

void SKGTableWithGraph::setGraphType(SKGTableWithGraph::GraphType iType) const
{
    ui.kDisplayMode->setCurrentIndex((int) iType);
}

SKGTableWithGraph::GraphType SKGTableWithGraph::getGraphType() const
{
    return (SKGTableWithGraph::GraphType) ui.kDisplayMode->currentIndex();
}

void SKGTableWithGraph::setGraphTypeSelectorVisible(bool iVisible) const
{
    ui.kGraphTypeFrm->setVisible(iVisible);
    if(m_actShowToolBar) m_actShowToolBar->setVisible(iVisible);
}

bool SKGTableWithGraph::isGraphTypeSelectorVisible() const
{
    return ui.kGraphTypeFrm->isVisible();
}

void SKGTableWithGraph::showText()
{
    showTable();

    ui.table_widget->hide();
    ui.kTextEdit->show();
}

void SKGTableWithGraph::showTable()
{
    ui.kTextEdit->hide();
    ui.table_widget2->show();
    if(m_scene) {
        m_scene->clear();
        delete m_scene;
    }
    m_scene = new SKGGraphicsScene();
    ui.graphicView->setScene(m_scene);
    ui.graph_widget->hide();

    ui.table_widget->show();
    m_graphVisible = false;
}

void SKGTableWithGraph::showGraph()
{
    ui.table_widget2->hide();
    ui.graph_widget->show();
    m_graphVisible = true;

    redrawGraphDelayed();
}

void SKGTableWithGraph::showTableAndGraph()
{
    showTable();
    ui.graph_widget->show();
    m_graphVisible = true;

    redrawGraphDelayed();
}

void SKGTableWithGraph::setSelectable(bool iSelectable)
{
    m_selectable = iSelectable;
}

bool SKGTableWithGraph::isSelectable() const
{
    return m_selectable;
}

#include "skgtablewithgraph.moc"

